# Generated by Django 3.2.4 on 2023-04-11 18:56

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('testrail_migrator', '0003_auto_20230324_0928'),
    ]

    operations = [
        migrations.RenameField(
            model_name='testrailsettings',
            old_name='api_url',
            new_name='testrail_api_url',
        ),
        migrations.RenameField(
            model_name='testrailsettings',
            old_name='preset_name',
            new_name='verbose_name',
        ),
    ]
