{% extends 'migrator_base.html' %}

{% block title %}{{ page_title }}{% endblock %}
{% block content %}
    <div class="py-3">
        <form method="post">
            {% csrf_token %}
            {{ form.as_p }}
            <button class="btn btn-primary">
                {{ button_label }}
            </button>
        </form>
    </div>
{% endblock %}