{% extends 'migrator_base.html' %}

{% block content %}
    <form method="post" id="configForm" class="py-3">
        <h2 class="py-2">Config Form</h2>
        {% csrf_token %}
        {{ form.as_p }}
        <button type="button" onclick="prettifyJson()" class="btn btn-outline-info">Prettify json</button>
    </form>
    <button type="submit" form="configForm" class="btn btn-primary">Save</button>
    <a class="ms-2" href="{% url 'plugins:testrail_migrator:settings-list' %}">Nevermind</a>
    <script>document.getElementById("id_custom_fields_matcher").addEventListener("load", prettifyJson());</script>
{% endblock %}